@@ -1,11 +1,11 @@
 package Tapper::Benchmark;
-# git description: v0.005-4-ga32373c
+# git description: v0.006-14-g62a0436
 
 BEGIN {
   $Tapper::Benchmark::AUTHORITY = 'cpan:TAPPER';
 }
 # ABSTRACT: Autonomous SQL backend to store benchmarks
-$Tapper::Benchmark::VERSION = '0.006';
+$Tapper::Benchmark::VERSION = '0.007';
 use strict;
 use warnings;
 
@@ -345,6 +345,70 @@ sub add_single_benchmark {
 
 }
 
+sub enqueue_multi_benchmark {
+
+    my ( $or_self, $ar_data_points, $hr_options ) = @_;
+
+    require Sereal::Encoder;
+
+    my $s_serialized = Sereal::Encoder->new({compress => 1})->encode($ar_data_points);
+    $or_self->{query}->insert_raw_bench_bundle($s_serialized);
+
+    return 1;
+
+}
+
+# dequeues a single bundle (can contain multiple data points)
+sub process_queued_multi_benchmark {
+
+    my ( $or_self, $hr_options ) = @_;
+
+    my $i_id;
+    my $s_serialized;
+    my $ar_data_points;
+    my $ar_results;
+    my $or_result;
+
+    # ===== exclusively pick single raw entry =====
+    # Lock single row via processing=1 so that only one worker handles it!
+    $or_self->{query}->start_transaction;
+    eval {
+            $ar_results = $or_self->{query}->select_raw_bench_bundle_for_lock;
+            $or_result  = $ar_results->fetchrow_hashref;
+            $i_id       = $or_result->{raw_bench_bundle_id};
+            goto RETURN unless $i_id;
+            $or_self->{query}->start_processing_raw_bench_bundle($i_id);
+    };
+    $or_self->{query}->finish_transaction( $@ );
+
+    # ===== process that single raw entry =====
+    $or_self->{query}->start_transaction;
+    eval {
+            require Sereal::Decoder;
+
+            $ar_results     = $or_self->{query}->select_raw_bench_bundle_for_processing($i_id);
+            $s_serialized   = $ar_results->fetchrow_hashref->{raw_bench_bundle_serialized};
+            $ar_data_points = Sereal::Decoder::decode_sereal($s_serialized);
+
+            # preserve order, otherwise add_multi_benchmark() would reorder to optimize insert
+            $or_self->add_multi_benchmark([$_], $hr_options) foreach @$ar_data_points;
+            $or_self->{query}->update_raw_bench_bundle_set_processed($i_id);
+    };
+    $or_self->{query}->finish_transaction( $@ );
+
+ RETURN:
+    return $@ ? undef : $i_id;
+
+}
+
+# garbage collect - initially raw_bench_bundles, but also other stuff.
+sub gc {
+
+    my ( $or_self, $hr_options ) = @_;
+
+    $or_self->{query}->delete_processed_raw_bench_bundles;
+}
+
 sub add_multi_benchmark {
 
     my ( $or_self, $ar_data_points, $hr_options ) = @_;
@@ -1097,7 +1161,7 @@ Every "key" create a new nested hash.
         ],
     });
 
-=head3 select_benchmark_point_essentials
+=head3 get_single_benchmark_point
 
 Get a single data points from the database including all essential
 fields (NAME, VALUE, UNIT) and additional fields.
@@ -1111,6 +1175,50 @@ Get a list of all benchmark NAMEs, optionally matching a given pattern
 
  $benchmarkanythingdata = $or_bench->list_benchmark_names($pattern);
 
+=head3 enqueue_multi_benchmark
+
+As a low-latency alternative to directly calling
+L</add_multi_benchmark> there is a queuing functionality.
+
+The C<enqueue_multi_benchmark> function simply writes the raw incoming
+data structure serialized (and compressed) into a single row and
+returns. The complementary function to this is
+C<process_queued_multi_benchmark> which takes these values over using
+the real C<add_multi_benchmark> internally.
+
+=head3 process_queued_multi_benchmark
+
+This is part 2 of the low-latency queuing alternative to directly
+calling L</add_multi_benchmark>.
+
+It transactionally marks a single raw entry as being processed and
+then takes over its values by calling C<add_multi_benchmark>. It
+preserves the order of entries by inserting each chunk sequentially,
+to not confuse the IDs to the careful reader. After the bundle is
+taken over it is marked as processed.
+
+This function only handles one single raw entry. It is expected to
+called from co-operating multiple worker tasks or multiple times from
+a wrapper.
+
+Currently the original raw values are B<not> deleted immediately, just
+for safety reasons, until the transactional code is death-proof (and
+certified by Stuntman Mike). There is a dedicated funtion L/gc> for
+that cleanup.
+
+The function returns the ID of the processed raw entry.
+
+=head3 gc
+
+This calls garbage collection, in particular deletes raw entries
+created by C<process_queued_multi_benchmark> and already processed by
+C<process_queued_multi_benchmark>.
+
+It is separated from those processing just for safety reasons until
+the transactional code in there is waterproof.
+
+The gc function can cleanup more stuff in the future.
+
 =head3 subsume
 
 This is a maintenance function for reducing the number of data points in the
