@@ -2,8 +2,8 @@ package Tapper::Benchmark::Query;
 BEGIN {
   $Tapper::Benchmark::Query::AUTHORITY = 'cpan:TAPPER';
 }
-# ABSTRACT: Tapper::Benchmark - qerying - base class
-$Tapper::Benchmark::Query::VERSION = '0.006';
+# ABSTRACT: Tapper::Benchmark - querying - base class
+$Tapper::Benchmark::Query::VERSION = '0.007';
 use strict;
 use warnings;
 use Digest::MD5;
@@ -70,6 +70,7 @@ sub start_transaction {
     local $or_self->{dbh}{RaiseError} = 1;
 
     eval {
+        push @{$or_self->{old_AutoCommit}}, $or_self->{dbh}{AutoCommit}; # allow nested transactions (does this make sense)
         $or_self->{dbh}{AutoCommit} = 0;
     };
     if ( $@ ) {
@@ -103,6 +104,9 @@ sub finish_transaction {
             $or_self->{dbh}->commit();
         }
 
+        my $old_AutoCommits = $or_self->{old_AutoCommit}; # reference, to change it
+        $or_self->{dbh}{AutoCommit} = pop @$old_AutoCommits if @$old_AutoCommits;
+
     }
 
     return 1;
@@ -119,7 +123,7 @@ __END__
 
 =head1 NAME
 
-Tapper::Benchmark::Query - Tapper::Benchmark - qerying - base class
+Tapper::Benchmark::Query - Tapper::Benchmark - querying - base class
 
 =head1 AUTHOR
 
