@@ -3,7 +3,7 @@ BEGIN {
   $Tapper::Benchmark::Query::SQLite::AUTHORITY = 'cpan:TAPPER';
 }
 # ABSTRACT: Tapper::Benchmark - qerying - SQLite backend
-$Tapper::Benchmark::Query::SQLite::VERSION = '0.006';
+$Tapper::Benchmark::Query::SQLite::VERSION = '0.007';
 use strict;
 use warnings;
 use feature 'switch';
@@ -711,6 +711,32 @@ sub select_addtyperelation {
 
 }
 
+sub select_raw_bench_bundle_for_lock {
+
+    my ( $or_self, @a_vals ) = @_;
+
+    return $or_self->execute_query( "
+        SELECT raw_bench_bundle_id
+        FROM raw_bench_bundles
+        WHERE processed=0 AND
+              processing=0
+        ORDER BY raw_bench_bundle_id ASC
+        LIMIT 1
+    ", @a_vals );
+}
+
+sub select_raw_bench_bundle_for_processing {
+
+    my ( $or_self, @a_vals ) = @_;
+
+    return $or_self->execute_query( "
+        SELECT raw_bench_bundle_serialized
+        FROM raw_bench_bundles
+        WHERE raw_bench_bundle_id = ?
+        LIMIT 1
+    ", @a_vals );
+}
+
 sub insert_addtyperelation {
 
     my ( $or_self, @a_vals ) = @_;
@@ -763,6 +789,18 @@ sub insert_benchmark_value {
 
 }
 
+sub insert_raw_bench_bundle {
+
+    my ( $or_self, @a_vals ) = @_;
+
+    return $or_self->execute_query( "
+        INSERT INTO raw_bench_bundles
+            (raw_bench_bundle_serialized)
+        VALUES ( ? )
+    ", @a_vals );
+
+}
+
 sub copy_additional_values {
 
     my ( $or_self, $hr_vals ) = @_;
@@ -892,6 +930,31 @@ sub update_benchmark_backup_value {
 
 }
 
+sub start_processing_raw_bench_bundle {
+
+    my ( $or_self, @a_vals ) = @_;
+
+    return $or_self->execute_query( "
+        UPDATE raw_bench_bundles
+        SET processing = 1
+        WHERE raw_bench_bundle_id = ?
+    ", @a_vals );
+
+}
+
+sub update_raw_bench_bundle_set_processed {
+
+    my ( $or_self, @a_vals ) = @_;
+
+    return $or_self->execute_query( "
+        UPDATE raw_bench_bundles
+        SET processed=1,
+            processing=0
+        WHERE raw_bench_bundle_id = ?
+    ", @a_vals );
+
+}
+
 sub delete_benchmark_additional_relations {
 
     my ( $or_self, @a_vals ) = @_;
@@ -914,6 +977,19 @@ sub delete_benchmark_value {
 
 }
 
+# Garbage Collection
+sub delete_processed_raw_bench_bundles {
+
+    my ( $or_self, @a_vals ) = @_;
+
+    return $or_self->execute_query( "
+        DELETE FROM raw_bench_bundles
+        WHERE processed=1 AND
+              processing=0
+    ", @a_vals );
+
+}
+
 1;
 
 __END__
